searchData={"items":[{"type":"module","title":"ExPmtiles","doc":"A module for working with PMTiles files.\n\nPMTiles is a single-file format for storing tiled map data. This module provides\nfunctionality to read and access tiles from PMTiles files stored either locally\nor on S3.","ref":"ExPmtiles.html"},{"type":"module","title":"Features - ExPmtiles","doc":"- Read PMTiles files from local storage or S3\n- Access tiles by zoom level and coordinates (z/x/y)\n- Automatic directory caching and decompression\n- Support for various compression types (gzip, none)\n- Tile ID calculations and conversions","ref":"ExPmtiles.html#module-features"},{"type":"module","title":"Usage - ExPmtiles","doc":"```elixir\n# Open a local PMTiles file\ninstance = ExPmtiles.new(\"path/to/file.pmtiles\", :local)\n\n# Open a PMTiles file from S3\ninstance = ExPmtiles.new(\"region\", \"my-bucket\", \"path/to/file.pmtiles\", :s3)\n\n# Get a tile by coordinates\ncase ExPmtiles.get_zxy(instance, 10, 512, 256) do\n  {{offset, length, data}, updated_instance} ->\n    # Use the tile data\n    data\n  {nil, updated_instance} ->\n    # Tile not found\n    nil\nend\n\n# Convert coordinates to tile ID\ntile_id = ExPmtiles.zxy_to_tile_id(10, 512, 256)\n\n# Convert tile ID back to coordinates\n{z, x, y} = ExPmtiles.tile_id_to_zxy(tile_id)\n```","ref":"ExPmtiles.html#module-usage"},{"type":"module","title":"Compression Support - ExPmtiles","doc":"The module supports the following compression types:\n- `:none` - No compression\n- `:gzip` - Gzip compression\n- `:unknown` - Unknown compression type\n\nNote: Brotli and Zstd compression are not currently supported.","ref":"ExPmtiles.html#module-compression-support"},{"type":"module","title":"Tile Types - ExPmtiles","doc":"Supported tile types include:\n- `:mvt` - Mapbox Vector Tiles\n- `:png` - PNG images\n- `:jpg` - JPEG images\n- `:webp` - WebP images\n- `:avif` - AVIF images\n- `:unknown` - Unknown tile type","ref":"ExPmtiles.html#module-tile-types"},{"type":"function","title":"ExPmtiles.decompress/2","doc":"Decompresses data based on the specified compression type.","ref":"ExPmtiles.html#decompress/2"},{"type":"function","title":"Parameters - ExPmtiles.decompress/2","doc":"- `data` - The compressed binary data\n- `compression_type` - The compression type (`:gzip`, `:none`, etc.)","ref":"ExPmtiles.html#decompress/2-parameters"},{"type":"function","title":"Returns - ExPmtiles.decompress/2","doc":"- `binary()` - The decompressed data","ref":"ExPmtiles.html#decompress/2-returns"},{"type":"function","title":"Raises - ExPmtiles.decompress/2","doc":"- Error for unsupported compression types (brotli, zstd)","ref":"ExPmtiles.html#decompress/2-raises"},{"type":"function","title":"Examples - ExPmtiles.decompress/2","doc":"iex> ExPmtiles.decompress(compressed_data, :gzip)\n    <<...>>\n\n    iex> ExPmtiles.decompress(data, :none)\n    data","ref":"ExPmtiles.html#decompress/2-examples"},{"type":"function","title":"ExPmtiles.deserialize_directory/1","doc":"Deserializes a PMTiles directory from binary data.\n\nPMTiles directories contain entries with tile IDs, offsets, lengths, and run lengths.\nThis function parses the binary format into a list of directory entries.","ref":"ExPmtiles.html#deserialize_directory/1"},{"type":"function","title":"Parameters - ExPmtiles.deserialize_directory/1","doc":"- `buf` - Binary data containing the directory","ref":"ExPmtiles.html#deserialize_directory/1-parameters"},{"type":"function","title":"Returns - ExPmtiles.deserialize_directory/1","doc":"- `list()` - List of directory entries with keys: `:tile_id`, `:offset`, `:length`, `:run_length`","ref":"ExPmtiles.html#deserialize_directory/1-returns"},{"type":"function","title":"Examples - ExPmtiles.deserialize_directory/1","doc":"iex> ExPmtiles.deserialize_directory(<<...>>)\n    [\n      %{tile_id: 0, offset: 0, length: 1024, run_length: 1},\n      %{tile_id: 1, offset: 1024, length: 512, run_length: 0}\n    ]","ref":"ExPmtiles.html#deserialize_directory/1-examples"},{"type":"function","title":"ExPmtiles.find_tile/2","doc":"Finds a tile entry in a directory by tile ID.\n\nUses binary search to efficiently locate the tile entry. Handles run-length\nencoding where multiple consecutive tiles may share the same entry.","ref":"ExPmtiles.html#find_tile/2"},{"type":"function","title":"Parameters - ExPmtiles.find_tile/2","doc":"- `entries` - List of directory entries\n- `tile_id` - The tile ID to search for","ref":"ExPmtiles.html#find_tile/2-parameters"},{"type":"function","title":"Returns - ExPmtiles.find_tile/2","doc":"- `map()` - The matching directory entry\n- `nil` - If no matching entry is found","ref":"ExPmtiles.html#find_tile/2-returns"},{"type":"function","title":"Examples - ExPmtiles.find_tile/2","doc":"iex> ExPmtiles.find_tile(entries, 1024)\n    %{tile_id: 1024, offset: 2048, length: 512, run_length: 1}","ref":"ExPmtiles.html#find_tile/2-examples"},{"type":"function","title":"ExPmtiles.get_bytes/3","doc":"Retrieves bytes from the PMTiles file using the configured storage backend.","ref":"ExPmtiles.html#get_bytes/3"},{"type":"function","title":"Parameters - ExPmtiles.get_bytes/3","doc":"- `instance` - The PMTiles instance\n- `offset` - Byte offset in the file\n- `length` - Number of bytes to read","ref":"ExPmtiles.html#get_bytes/3-parameters"},{"type":"function","title":"Returns - ExPmtiles.get_bytes/3","doc":"- `binary()` - The requested bytes\n- `nil` - If the bytes cannot be read","ref":"ExPmtiles.html#get_bytes/3-returns"},{"type":"function","title":"Examples - ExPmtiles.get_bytes/3","doc":"iex> ExPmtiles.get_bytes(instance, 0, 1024)\n    <<...>>","ref":"ExPmtiles.html#get_bytes/3-examples"},{"type":"function","title":"ExPmtiles.get_tile/3","doc":"Gets raw tile data from the PMTiles file at the specified offset and length.","ref":"ExPmtiles.html#get_tile/3"},{"type":"function","title":"Parameters - ExPmtiles.get_tile/3","doc":"- `instance` - The PMTiles instance\n- `offset` - Byte offset in the file\n- `length` - Number of bytes to read","ref":"ExPmtiles.html#get_tile/3-parameters"},{"type":"function","title":"Returns - ExPmtiles.get_tile/3","doc":"- `binary()` - The raw tile data\n- `nil` - If the data cannot be read","ref":"ExPmtiles.html#get_tile/3-returns"},{"type":"function","title":"Examples - ExPmtiles.get_tile/3","doc":"iex> ExPmtiles.get_tile(instance, 1024, 512)\n    <<...>>","ref":"ExPmtiles.html#get_tile/3-examples"},{"type":"function","title":"ExPmtiles.get_zxy/4","doc":"Gets a tile by its zoom level and coordinates.\n\nThis function converts the z/x/y coordinates to a tile ID and searches for the\ntile in the PMTiles directory structure. It handles both leaf and internal\ndirectories automatically.","ref":"ExPmtiles.html#get_zxy/4"},{"type":"function","title":"Parameters - ExPmtiles.get_zxy/4","doc":"- `instance` - The PMTiles instance\n- `z` - Zoom level (integer)\n- `x` - X coordinate (integer)\n- `y` - Y coordinate (integer)","ref":"ExPmtiles.html#get_zxy/4-parameters"},{"type":"function","title":"Returns - ExPmtiles.get_zxy/4","doc":"- `{{offset, length, data}, updated_instance}` - Tuple containing tile information and updated instance\n- `{nil, updated_instance}` - If tile is not found or outside zoom bounds","ref":"ExPmtiles.html#get_zxy/4-returns"},{"type":"function","title":"Examples - ExPmtiles.get_zxy/4","doc":"iex> ExPmtiles.get_zxy(instance, 10, 512, 256)\n    {{1024, 512, <<...>>}, updated_instance}\n\n    iex> ExPmtiles.get_zxy(instance, 25, 0, 0)\n    {nil, instance}  # Zoom level out of bounds","ref":"ExPmtiles.html#get_zxy/4-examples"},{"type":"function","title":"ExPmtiles.new/2","doc":"Creates a new PMTiles instance for a locally-stored file.","ref":"ExPmtiles.html#new/2"},{"type":"function","title":"Parameters - ExPmtiles.new/2","doc":"- `path` - The local file path to the PMTiles file\n- `:local` - Source type identifier","ref":"ExPmtiles.html#new/2-parameters"},{"type":"function","title":"Returns - ExPmtiles.new/2","doc":"- `%ExPmtiles{}` - A configured PMTiles instance with parsed header\n- `nil` - If the file cannot be accessed or is invalid","ref":"ExPmtiles.html#new/2-returns"},{"type":"function","title":"Examples - ExPmtiles.new/2","doc":"iex> ExPmtiles.new(\"data/world.pmtiles\", :local)\n    %ExPmtiles{bucket: nil, path: \"data/world.pmtiles\", source: :local, header: %{...}}","ref":"ExPmtiles.html#new/2-examples"},{"type":"function","title":"ExPmtiles.new/4","doc":"Creates a new PMTiles instance for an S3-stored file.","ref":"ExPmtiles.html#new/4"},{"type":"function","title":"Parameters - ExPmtiles.new/4","doc":"- `bucket` - The S3 bucket name\n- `path` - The path to the PMTiles file within the bucket\n- `:s3` - Source type identifier","ref":"ExPmtiles.html#new/4-parameters"},{"type":"function","title":"Returns - ExPmtiles.new/4","doc":"- `%ExPmtiles{}` - A configured PMTiles instance with parsed header\n- `nil` - If the file cannot be accessed or is invalid","ref":"ExPmtiles.html#new/4-returns"},{"type":"function","title":"Examples - ExPmtiles.new/4","doc":"iex> ExPmtiles.new(\"my-bucket\", \"maps/world.pmtiles\", :s3)\n    %ExPmtiles{bucket: \"my-bucket\", path: \"maps/world.pmtiles\", source: :s3, header: %{...}}","ref":"ExPmtiles.html#new/4-examples"},{"type":"function","title":"ExPmtiles.parse_header/1","doc":"Parses the PMTiles file header from binary data.\n\nThe PMTiles header contains metadata about the file including offsets,\ncompression settings, zoom levels, and geographic bounds.","ref":"ExPmtiles.html#parse_header/1"},{"type":"function","title":"Parameters - ExPmtiles.parse_header/1","doc":"- `binary` - Binary data containing the PMTiles header (first 16,384 bytes)","ref":"ExPmtiles.html#parse_header/1-parameters"},{"type":"function","title":"Returns - ExPmtiles.parse_header/1","doc":"- `map()` - Header information with keys:\n  - `:magic_number` - File magic number (\"PMTiles\")\n  - `:spec_version` - PMTiles specification version\n  - `:root_offset`, `:root_length` - Root directory location\n  - `:metadata_offset`, `:metadata_length` - Metadata location\n  - `:leaf_dir_offset`, `:leaf_dir_length` - Leaf directories location\n  - `:tile_data_offset`, `:tile_data_length` - Tile data location\n  - `:num_addr_tiles`, `:num_tile_entries`, `:num_tile_contents` - Tile counts\n  - `:clustered?` - Whether tiles are clustered\n  - `:internal_compression`, `:tile_compression` - Compression types\n  - `:tile_type` - Type of tiles stored\n  - `:min_zoom`, `:max_zoom` - Zoom level bounds\n  - `:min_position`, `:max_position` - Geographic bounds\n  - `:center_zoom`, `:center_position` - Center point","ref":"ExPmtiles.html#parse_header/1-returns"},{"type":"function","title":"Examples - ExPmtiles.parse_header/1","doc":"iex> ExPmtiles.parse_header(header_binary)\n    %{\n      magic_number: \"PMTiles\",\n      spec_version: 3,\n      min_zoom: 0,\n      max_zoom: 14,\n      ...\n    }","ref":"ExPmtiles.html#parse_header/1-examples"},{"type":"function","title":"ExPmtiles.read_varint/3","doc":"Reads a variable-length integer from binary data.\n\nPMTiles uses variable-length integers (varints) for efficient encoding of\nsmall numbers. This function reads one varint from the beginning of the binary.","ref":"ExPmtiles.html#read_varint/3"},{"type":"function","title":"Parameters - ExPmtiles.read_varint/3","doc":"- `binary` - Binary data containing the varint\n- `shift` - Internal parameter for bit shifting (default: 0)\n- `result` - Internal parameter for accumulating result (default: 0)","ref":"ExPmtiles.html#read_varint/3-parameters"},{"type":"function","title":"Returns - ExPmtiles.read_varint/3","doc":"- `{integer(), binary()}` - Tuple of the decoded integer and remaining binary\n- Raises error if binary is empty or malformed","ref":"ExPmtiles.html#read_varint/3-returns"},{"type":"function","title":"Examples - ExPmtiles.read_varint/3","doc":"iex> ExPmtiles.read_varint(<<1>>)\n    {1, \"\"}\n\n    iex> ExPmtiles.read_varint(<<128, 1>>)\n    {128, \"\"}","ref":"ExPmtiles.html#read_varint/3-examples"},{"type":"function","title":"ExPmtiles.tile_id_to_zxy/1","doc":"Converts a PMTiles tile ID back to zoom level and coordinates.\n\nPerforms the inverse operation of `zxy_to_tile_id/3`, converting a tile ID\nback to its corresponding z/x/y coordinates.","ref":"ExPmtiles.html#tile_id_to_zxy/1"},{"type":"function","title":"Parameters - ExPmtiles.tile_id_to_zxy/1","doc":"- `tile_id` - The tile ID to convert","ref":"ExPmtiles.html#tile_id_to_zxy/1-parameters"},{"type":"function","title":"Returns - ExPmtiles.tile_id_to_zxy/1","doc":"- `{z, x, y}` - Tuple of zoom level and coordinates","ref":"ExPmtiles.html#tile_id_to_zxy/1-returns"},{"type":"function","title":"Raises - ExPmtiles.tile_id_to_zxy/1","doc":"- `ArgumentError` - If tile_id is invalid or exceeds 64-bit limit","ref":"ExPmtiles.html#tile_id_to_zxy/1-raises"},{"type":"function","title":"Examples - ExPmtiles.tile_id_to_zxy/1","doc":"iex> ExPmtiles.tile_id_to_zxy(1048576)\n    {10, 512, 256}\n\n    iex> ExPmtiles.tile_id_to_zxy(0)\n    {0, 0, 0}","ref":"ExPmtiles.html#tile_id_to_zxy/1-examples"},{"type":"function","title":"ExPmtiles.zxy_to_tile_id/3","doc":"Converts zoom level and coordinates to a PMTiles tile ID.\n\nUses the Hilbert curve mapping to convert z/x/y coordinates to a unique tile ID.\nThis is the standard method used by PMTiles for tile identification.","ref":"ExPmtiles.html#zxy_to_tile_id/3"},{"type":"function","title":"Parameters - ExPmtiles.zxy_to_tile_id/3","doc":"- `z` - Zoom level (integer, 0-26)\n- `x` - X coordinate (integer, 0 to 2^z - 1)\n- `y` - Y coordinate (integer, 0 to 2^z - 1)","ref":"ExPmtiles.html#zxy_to_tile_id/3-parameters"},{"type":"function","title":"Returns - ExPmtiles.zxy_to_tile_id/3","doc":"- `integer()` - The tile ID","ref":"ExPmtiles.html#zxy_to_tile_id/3-returns"},{"type":"function","title":"Raises - ExPmtiles.zxy_to_tile_id/3","doc":"- `ArgumentError` - If zoom level exceeds 26 or coordinates are out of bounds","ref":"ExPmtiles.html#zxy_to_tile_id/3-raises"},{"type":"function","title":"Examples - ExPmtiles.zxy_to_tile_id/3","doc":"iex> ExPmtiles.zxy_to_tile_id(10, 512, 256)\n    1048576\n\n    iex> ExPmtiles.zxy_to_tile_id(0, 0, 0)\n    0","ref":"ExPmtiles.html#zxy_to_tile_id/3-examples"},{"type":"behaviour","title":"ExPmtiles.Behaviour","doc":"A behaviour module for PMTiles files. Primarily to support Mox testing.","ref":"ExPmtiles.Behaviour.html"},{"type":"callback","title":"ExPmtiles.Behaviour.get_tile/3","doc":"","ref":"ExPmtiles.Behaviour.html#c:get_tile/3"},{"type":"callback","title":"ExPmtiles.Behaviour.get_zxy/4","doc":"","ref":"ExPmtiles.Behaviour.html#c:get_zxy/4"},{"type":"callback","title":"ExPmtiles.Behaviour.new/2","doc":"","ref":"ExPmtiles.Behaviour.html#c:new/2"},{"type":"callback","title":"ExPmtiles.Behaviour.new/4","doc":"","ref":"ExPmtiles.Behaviour.html#c:new/4"},{"type":"callback","title":"ExPmtiles.Behaviour.tile_id_to_zxy/1","doc":"","ref":"ExPmtiles.Behaviour.html#c:tile_id_to_zxy/1"},{"type":"callback","title":"ExPmtiles.Behaviour.zxy_to_tile_id/3","doc":"","ref":"ExPmtiles.Behaviour.html#c:zxy_to_tile_id/3"},{"type":"module","title":"ExPmtiles.Cache","doc":"A GenServer implementation of a multi-level cache for PMTiles file data.\n\nThis GenServer provides efficient access to PMTiles map data through several caching mechanisms:","ref":"ExPmtiles.Cache.html"},{"type":"module","title":"Cache Levels - ExPmtiles.Cache","doc":"","ref":"ExPmtiles.Cache.html#module-cache-levels"},{"type":"module","title":"1. Directory Cache - ExPmtiles.Cache","doc":"- Caches deserialized directory structures from the PMTiles file\n- Pre-populates cache for zoom levels 0-4 on startup\n- Persists cached directories to disk for faster restarts\n- Uses zlib compression for efficient storage\n- Shared across all requests to prevent duplicate S3 fetches","ref":"ExPmtiles.Cache.html#module-1-directory-cache"},{"type":"module","title":"2. Tile Cache - ExPmtiles.Cache","doc":"- Uses ETS tables to cache individual tile data\n- Implements LRU (Least Recently Used) eviction\n- Configurable maximum entries (default: 100,000)\n- Maintains hit/miss statistics\n- Automatic cleanup of idle entries (1 hour timeout)","ref":"ExPmtiles.Cache.html#module-2-tile-cache"},{"type":"module","title":"3. Concurrent Request Handling - ExPmtiles.Cache","doc":"- Prevents duplicate requests for the same tile\n- Coordinates multiple requests through a pending requests table\n- Broadcasts results to all waiting processes\n- 30-second timeout for waiting processes","ref":"ExPmtiles.Cache.html#module-3-concurrent-request-handling"},{"type":"module","title":"Cache Files - ExPmtiles.Cache","doc":"Directory cache files are stored in `priv/pmtiles_cache` with the format:\n`{bucket}_{filename}_directories.cache`","ref":"ExPmtiles.Cache.html#module-cache-files"},{"type":"module","title":"Configuration - ExPmtiles.Cache","doc":"- `:max_entries` - Maximum number of tiles to cache (default: 100,000)\n- `:pmtiles_module` - Module to use for PMTiles operations (configurable for testing)","ref":"ExPmtiles.Cache.html#module-configuration"},{"type":"module","title":"Usage - ExPmtiles.Cache","doc":"```elixir\n# Start the cache for an S3 PMTiles file\n{:ok, pid} = ExPmtiles.Cache.start_link(bucket: \"maps\", path: \"map.pmtiles\")\n\n# Start the cache for a local PMTiles file\n{:ok, pid} = ExPmtiles.Cache.start_link(bucket: nil, path: \"/path/to/map.pmtiles\")\n\n# Get a tile by coordinates\ncase ExPmtiles.Cache.get_tile(pid, 10, 512, 256) do\n  {:ok, tile_data} ->\n    # Handle tile data\n    tile_data\n  {:error, reason} ->\n    # Handle error\n    nil\nend\n\n# Get cache statistics\nstats = ExPmtiles.Cache.get_stats(pid)\n# Returns: %{hits: 150, misses: 25}\n```","ref":"ExPmtiles.Cache.html#module-usage"},{"type":"module","title":"Implementation Details - ExPmtiles.Cache","doc":"The GenServer maintains several ETS tables:\n- **Main table**: Caches tile data with timestamps for LRU eviction\n- **Pending table**: Tracks in-progress requests to prevent duplicates\n- **Stats table**: Maintains hit/miss statistics\n\nThe directory cache is maintained in the GenServer state and is progressively\nupdated as new directories are accessed. This prevents duplicate S3 requests\nand directory deserialization operations.","ref":"ExPmtiles.Cache.html#module-implementation-details"},{"type":"module","title":"Performance Features - ExPmtiles.Cache","doc":"- **Background population**: Pre-loads directories for zoom levels 0-4\n- **Persistent cache**: Saves directories to disk for faster restarts\n- **Compression**: Uses zlib compression for cache files\n- **Concurrent access**: Multiple processes can safely access the same cache\n- **Automatic cleanup**: Removes idle entries every 5 minutes","ref":"ExPmtiles.Cache.html#module-performance-features"},{"type":"function","title":"ExPmtiles.Cache.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"ExPmtiles.Cache.html#child_spec/1"},{"type":"function","title":"ExPmtiles.Cache.get_stats/1","doc":"Retrieves cache statistics.\n\nReturns hit and miss counts for the cache, useful for monitoring cache\nperformance and effectiveness.","ref":"ExPmtiles.Cache.html#get_stats/1"},{"type":"function","title":"Parameters - ExPmtiles.Cache.get_stats/1","doc":"- `pid` - The cache process PID","ref":"ExPmtiles.Cache.html#get_stats/1-parameters"},{"type":"function","title":"Returns - ExPmtiles.Cache.get_stats/1","doc":"- `%{hits: integer(), misses: integer()}` - Cache statistics","ref":"ExPmtiles.Cache.html#get_stats/1-returns"},{"type":"function","title":"Examples - ExPmtiles.Cache.get_stats/1","doc":"iex> {:ok, pid} = ExPmtiles.Cache.start_link(bucket: \"maps\", path: \"world.pmtiles\")\n    iex> ExPmtiles.Cache.get_stats(pid)\n    %{hits: 150, misses: 25}\n\n    iex> # After some cache operations\n    iex> ExPmtiles.Cache.get_stats(pid)\n    %{hits: 175, misses: 30}","ref":"ExPmtiles.Cache.html#get_stats/1-examples"},{"type":"function","title":"ExPmtiles.Cache.get_tile/4","doc":"Retrieves a tile from the cache by coordinates.\n\nThis function handles both cache hits and misses. If the tile is not in the cache,\nit will be fetched from the PMTiles file and cached for future requests.","ref":"ExPmtiles.Cache.html#get_tile/4"},{"type":"function","title":"Parameters - ExPmtiles.Cache.get_tile/4","doc":"- `{bucket, path}` - Tuple identifying the PMTiles file or `pid` identifying the cache process\n- `z` - Zoom level (integer)\n- `x` - X coordinate (integer)\n- `y` - Y coordinate (integer)","ref":"ExPmtiles.Cache.html#get_tile/4-parameters"},{"type":"function","title":"Returns - ExPmtiles.Cache.get_tile/4","doc":"- `{:ok, tile_data}` - Tile data as binary\n- `{:error, reason}` - Error retrieving tile (e.g., `:tile_not_found`, `:timeout`)\n- `nil` - If the cache process is not available","ref":"ExPmtiles.Cache.html#get_tile/4-returns"},{"type":"function","title":"Examples - ExPmtiles.Cache.get_tile/4","doc":"iex> ExPmtiles.Cache.get_tile({\"maps\", \"world.pmtiles\"}, 10, 512, 256)\n    {:ok, <<...>>}\n\n    iex> ExPmtiles.Cache.get_tile({\"maps\", \"world.pmtiles\"}, 25, 0, 0)\n    {:error, :tile_not_found}\n\n    iex> ExPmtiles.Cache.get_tile(pid, 10, 512, 256)\n    {:ok, <<...>>}","ref":"ExPmtiles.Cache.html#get_tile/4-examples"},{"type":"function","title":"ExPmtiles.Cache.start_link/1","doc":"Starts a new PMTiles cache GenServer.\n\nCreates a named GenServer process that manages caching for a specific PMTiles file.\nThe process name is derived from the bucket and path to ensure uniqueness.","ref":"ExPmtiles.Cache.html#start_link/1"},{"type":"function","title":"Parameters - ExPmtiles.Cache.start_link/1","doc":"- `opts` - Keyword list of options:\n  - `:bucket` - S3 bucket name (or `nil` for local files)\n  - `:path` - Path to the PMTiles file\n  - `:max_entries` - Maximum number of tiles to cache (default: 100,000)","ref":"ExPmtiles.Cache.html#start_link/1-parameters"},{"type":"function","title":"Returns - ExPmtiles.Cache.start_link/1","doc":"- `{:ok, pid}` - Successfully started cache process\n- `{:error, reason}` - Failed to start cache","ref":"ExPmtiles.Cache.html#start_link/1-returns"},{"type":"function","title":"Examples - ExPmtiles.Cache.start_link/1","doc":"iex> ExPmtiles.Cache.start_link(region: nil, bucket: \"maps\", path: \"world.pmtiles\")\n    {:ok, #PID<0.123.0>}\n\n    iex> ExPmtiles.Cache.start_link(bucket: nil, path: \"/data/local.pmtiles\")\n    {:ok, #PID<0.124.0>}\n\n    iex> ExPmtiles.Cache.start_link(bucket: \"maps\", path: \"world.pmtiles\", max_entries: 50_000)\n    {:ok, #PID<0.125.0>}","ref":"ExPmtiles.Cache.html#start_link/1-examples"},{"type":"module","title":"ExPmtiles.Storage","doc":"Storage adapter for PMTiles files, supporting both S3 and local file storage.\n\nThis module provides a unified interface for reading bytes from PMTiles files\nregardless of whether they are stored locally or in Amazon S3. It handles\nthe different access patterns and error conditions for each storage type.","ref":"ExPmtiles.Storage.html"},{"type":"module","title":"Storage Types - ExPmtiles.Storage","doc":"","ref":"ExPmtiles.Storage.html#module-storage-types"},{"type":"module","title":"S3 Storage - ExPmtiles.Storage","doc":"- Uses ExAws for S3 access\n- Supports range requests for efficient partial file reading\n- Configurable timeouts and connection pooling\n- Automatic error handling and logging","ref":"ExPmtiles.Storage.html#module-s3-storage"},{"type":"module","title":"Local Storage - ExPmtiles.Storage","doc":"- Direct file system access\n- Binary file reading with offset support\n- Automatic file handle management\n- Graceful error handling for missing files","ref":"ExPmtiles.Storage.html#module-local-storage"},{"type":"module","title":"Usage - ExPmtiles.Storage","doc":"```elixir\n# The module is used internally by ExPmtiles\n# You typically don't call these functions directly\n\n# For S3 files\ninstance = %ExPmtiles{source: :s3, bucket: \"my-bucket\", path: \"file.pmtiles\"}\ndata = ExPmtiles.Storage.get_bytes(instance, 0, 1024)\n\n# For local files\ninstance = %ExPmtiles{source: :local, path: \"/path/to/file.pmtiles\"}\ndata = ExPmtiles.Storage.get_bytes(instance, 0, 1024)\n```","ref":"ExPmtiles.Storage.html#module-usage"},{"type":"module","title":"Configuration - ExPmtiles.Storage","doc":"S3 requests use the following default configuration:\n- Receive timeout: 30 seconds\n- Connection pool: `:s3_pool`\n- Range request support for partial reads","ref":"ExPmtiles.Storage.html#module-configuration"},{"type":"function","title":"ExPmtiles.Storage.get_bytes/3","doc":"Retrieves a range of bytes from a PMTiles file.\n\nThis function automatically routes to the appropriate storage backend based on\nthe instance's source type. It provides a unified interface for reading file\ndata regardless of storage location.","ref":"ExPmtiles.Storage.html#get_bytes/3"},{"type":"function","title":"Parameters - ExPmtiles.Storage.get_bytes/3","doc":"- `instance` - The PMTiles instance containing storage configuration\n- `offset` - Byte offset in the file (0-based)\n- `length` - Number of bytes to read","ref":"ExPmtiles.Storage.html#get_bytes/3-parameters"},{"type":"function","title":"Returns - ExPmtiles.Storage.get_bytes/3","doc":"- `binary()` - The requested bytes\n- `nil` - If the bytes cannot be read (file not found, network error, etc.)","ref":"ExPmtiles.Storage.html#get_bytes/3-returns"},{"type":"function","title":"Examples - ExPmtiles.Storage.get_bytes/3","doc":"iex> instance = %ExPmtiles{source: :local, path: \"test.pmtiles\"}\n    iex> ExPmtiles.Storage.get_bytes(instance, 0, 1024)\n    <<...>>\n\n    iex> instance = %ExPmtiles{source: :s3, bucket: \"bucket\", path: \"file.pmtiles\"}\n    iex> ExPmtiles.Storage.get_bytes(instance, 1024, 512)\n    <<...>>","ref":"ExPmtiles.Storage.html#get_bytes/3-examples"},{"type":"extras","title":"ExPmtiles","doc":"# ExPmtiles\n\nAn Elixir library for working with PMTiles files - a single-file format for storing tiled map data.","ref":"readme.html"},{"type":"extras","title":"Overview - ExPmtiles","doc":"PMTiles is an efficient single-file format for storing tiled map data, designed for cloud storage and CDN delivery. This library provides a complete Elixir implementation for reading and accessing tiles from PMTiles files stored either locally or on Amazon S3.","ref":"readme.html#overview"},{"type":"extras","title":"Features - ExPmtiles","doc":"- **Multi-storage support**: Read PMTiles files from local storage or Amazon S3\n- **Efficient caching**: Multi-level caching system with directory and tile caching\n- **Concurrent access**: Safe concurrent access with request deduplication\n- **Compression support**: Built-in support for gzip compression\n- **Tile type support**: MVT, PNG, JPEG, WebP, and AVIF tile formats\n- **Performance optimized**: Background directory pre-loading and persistent cache storage\n- **Production ready**: Configurable timeouts, connection pooling, and error handling","ref":"readme.html#features"},{"type":"extras","title":"Installation - ExPmtiles","doc":"Add `ex_pmtiles` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:ex_pmtiles, \"~> 0.1.0\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Quick Start - ExPmtiles","doc":"","ref":"readme.html#quick-start"},{"type":"extras","title":"Basic Usage - ExPmtiles","doc":"```elixir\n# Open a local PMTiles file\ninstance = ExPmtiles.new(\"path/to/file.pmtiles\", :local)\n\n# Open a PMTiles file from S3\ninstance = ExPmtiles.new(\"my-bucket\", \"path/to/file.pmtiles\", :s3)\n\n# Get a tile by coordinates\ncase ExPmtiles.get_zxy(instance, 10, 512, 256) do\n  {{offset, length, data}, updated_instance} ->\n    # Use the tile data\n    data\n  {nil, updated_instance} ->\n    # Tile not found\n    nil\nend\n```","ref":"readme.html#basic-usage"},{"type":"extras","title":"Using the Cache - ExPmtiles","doc":"For production applications, use the caching layer for better performance:\n\n```elixir\n# Start the cache for an S3 PMTiles file\n{:ok, cache_pid} = ExPmtiles.Cache.start_link(\n  bucket: \"maps\", \n  path: \"world.pmtiles\",\n  max_entries: 100_000\n)\n\n# Get tiles with automatic caching\ncase ExPmtiles.Cache.get_tile(cache_pid, 10, 512, 256) do\n  {:ok, tile_data} ->\n    # Handle tile data\n    tile_data\n  {:error, reason} ->\n    # Handle error\n    nil\nend\n\n# Get cache statistics\nstats = ExPmtiles.Cache.get_stats(cache_pid)\n# Returns: %{hits: 150, misses: 25}\n```","ref":"readme.html#using-the-cache"},{"type":"extras","title":"API Reference - ExPmtiles","doc":"","ref":"readme.html#api-reference"},{"type":"extras","title":"Core Functions - ExPmtiles","doc":"#### `ExPmtiles.new/2` and `ExPmtiles.new/4`\n\nCreate a new PMTiles instance:\n\n```elixir\n# Local file\ninstance = ExPmtiles.new(\"data/world.pmtiles\", :local)\n\n# S3 file\ninstance = ExPmtiles.new(\"my-bucket\", \"maps/world.pmtiles\", :s3)\n```\n\n#### `ExPmtiles.get_zxy/4`\n\nGet a tile by zoom level and coordinates:\n\n```elixir\ncase ExPmtiles.get_zxy(instance, 10, 512, 256) do\n  {{offset, length, data}, updated_instance} ->\n    # Tile found\n    data\n  {nil, updated_instance} ->\n    # Tile not found\n    nil\nend\n```\n\n#### `ExPmtiles.zxy_to_tile_id/3` and `ExPmtiles.tile_id_to_zxy/1`\n\nConvert between coordinates and tile IDs:\n\n```elixir\n# Convert coordinates to tile ID\ntile_id = ExPmtiles.zxy_to_tile_id(10, 512, 256)\n\n# Convert tile ID back to coordinates\n{z, x, y} = ExPmtiles.tile_id_to_zxy(tile_id)\n```","ref":"readme.html#core-functions"},{"type":"extras","title":"Cache Functions - ExPmtiles","doc":"#### `ExPmtiles.Cache.start_link/1`\n\nStart a cache process:\n\n```elixir\n{:ok, pid} = ExPmtiles.Cache.start_link(\n  bucket: \"maps\",\n  path: \"world.pmtiles\",\n  max_entries: 100_000\n)\n```\n\n#### `ExPmtiles.Cache.get_tile/4`\n\nGet a tile with caching:\n\n```elixir\ncase ExPmtiles.Cache.get_tile(pid, 10, 512, 256) do\n  {:ok, data} -> data\n  {:error, reason} -> nil\nend\n```","ref":"readme.html#cache-functions"},{"type":"extras","title":"Configuration - ExPmtiles","doc":"","ref":"readme.html#configuration"},{"type":"extras","title":"S3 Configuration - ExPmtiles","doc":"For S3 access, ensure you have ExAws configured in your application:\n\n```elixir\n# In config/config.exs\nconfig :ex_aws,\n  access_key_id: {:system, \"AWS_ACCESS_KEY_ID\"},\n  secret_access_key: {:system, \"AWS_SECRET_ACCESS_KEY\"},\n  region: \"us-east-1\"\n```","ref":"readme.html#s3-configuration"},{"type":"extras","title":"Cache Configuration - ExPmtiles","doc":"Configure cache behavior:\n\n```elixir\n# In config/config.exs\nconfig :ex_pmtiles,\n  max_entries: 100_000,  # Maximum tiles to cache\n  cache_dir: \"priv/pmtiles_cache\"  # Directory for persistent cache\n```","ref":"readme.html#cache-configuration"},{"type":"extras","title":"Performance Features - ExPmtiles","doc":"","ref":"readme.html#performance-features"},{"type":"extras","title":"Multi-Level Caching - ExPmtiles","doc":"The library implements a sophisticated caching system:\n\n1. **Directory Cache**: Caches deserialized PMTiles directory structures\n2. **Tile Cache**: Caches individual tile data with LRU eviction\n3. **Persistent Cache**: Saves directories to disk for faster restarts","ref":"readme.html#multi-level-caching"},{"type":"extras","title":"Background Pre-loading - ExPmtiles","doc":"The cache automatically pre-loads directories for zoom levels 0-4 in the background, improving response times for common zoom levels.","ref":"readme.html#background-pre-loading"},{"type":"extras","title":"Concurrent Request Handling - ExPmtiles","doc":"Multiple processes can safely access the same cache without duplicate requests. The system coordinates requests and broadcasts results to all waiting processes.","ref":"readme.html#concurrent-request-handling"},{"type":"extras","title":"Supported Formats - ExPmtiles","doc":"","ref":"readme.html#supported-formats"},{"type":"extras","title":"Compression Types - ExPmtiles","doc":"- `:none` - No compression\n- `:gzip` - Gzip compression\n- `:unknown` - Unknown compression type","ref":"readme.html#compression-types"},{"type":"extras","title":"Tile Types - ExPmtiles","doc":"- `:mvt` - Mapbox Vector Tiles\n- `:png` - PNG images\n- `:jpg` - JPEG images\n- `:webp` - WebP images\n- `:avif` - AVIF images","ref":"readme.html#tile-types"},{"type":"extras","title":"Testing - ExPmtiles","doc":"The library includes test support with mock implementations:\n\n```elixir\n# In test/test_helper.exs\nMox.defmock(ExPmtiles.CacheMock, for: ExPmtiles.Behaviour)\n\n# Mock implementations are automatically configured for test environment\n```","ref":"readme.html#testing"},{"type":"extras","title":"Contributing - ExPmtiles","doc":"1. Fork the repository\n2. Create a feature branch\n3. Make your changes\n4. Add tests for new functionality\n5. Ensure all tests pass\n6. Submit a pull request","ref":"readme.html#contributing"},{"type":"extras","title":"License - ExPmtiles","doc":"This project is licensed under the MIT License - see the LICENSE file for details.","ref":"readme.html#license"},{"type":"extras","title":"Documentation - ExPmtiles","doc":"Documentation can be generated with [ExDoc](https://github.com/elixir-lang/ex_doc) and published on [HexDocs](https://hexdocs.pm). Once published, the docs can be found at  .","ref":"readme.html#documentation"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.38.2"}}